/* PhotoProc - user interface for basic photo editing.
 *
 * This file is part of PhotoPNMTools,
 * Release $Name:  $,
 * File rev. $Id: PhotoProc.java,v 1.6 2003/03/08 17:17:21 schooten Exp $.
 *
 * Copyright (c) 2002, 2003 by B.W. van Schooten. All rights reserved.
 * This software is distributed under the Perl Artistic License.
 * No warranty. See file 'artistic.license' for more details.
 *
 * boris@13thmonkey.org
 * www.13thmonkey.org/~boris/photopnmtools/
 */

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;
public class PhotoProc {
	static Dimension imgsize;
	static Frame cropp = new Frame();
	static ImageCanvas croppc;
	static Frame hsyp = new Frame();
	static ImageCanvas hsypc;

	static String hue [] = 
		new String [] {"red","yellow","green", "cyan","blue","magenta"};

	static Panel crop = new Panel();
	static Scrollbar x1,y1,x2,y2;
	static int lcx1,lcy1,lcx2,lcy2;
	static Panel hsy = new Panel();
	static Scrollbar s,y,Y,contrast,r,g,b,n,whiten;
	static Scrollbar [] hs = new Scrollbar[6];
	static Label slab=new Label(),ylab=new Label(),Ylab=new Label();
	static Label contrastlab=new Label(),rlab=new Label(),glab=new Label();
	static Label blab=new Label(),nlab=new Label(),whitenlab=new Label();
	static Label [] hslab = new Label [] 
	{new Label(),new Label(),new Label(),new Label(),new Label(),new Label()};

	static Button rotleftbut= new Button("ROTATE LEFT");
	static Button rotrightbut = new Button("ROTATE RIGHT");
	static Button norotbut = new Button("NO ROTATE");
	static Button cropresetbut = new Button("RESET");
	static Button hsyresetbut = new Button("RESET");
	static Button donebut = new Button("DONE");
	static Button sharpbut = new Button("SHOW SHARPENED");

	static Label cropf1 = new Label("LEFT        <<<<<<<<<<<");
	static Label cropf2 = new Label("UP          ^^^^^^^^^^^");
	static Label cropf3 = new Label("RIGHT       >>>>>>>>>>>");
	static Label cropf4 = new Label("DOWN        VVVVVVVVVVV");

	static Button cropupdbut = new Button("UPDATE DISPLAY");
	static Button hsyupdbut = new Button("UPDATE");

	static String origfile,filenopath,basename;

	static String source="photoproctemp0.ppm";
	static String cropprv="photoproctemp1.ppm",
	 hsyprv="photoproctemp2.ppm";

	static int rotstate=0;

	static CmdExecuter exec;

	static boolean is_preview=true;
	static boolean sharpen=false;

	//static String baseurl = "file:///./";
	static String baseurl = "file://";

	static Image loadImage(String imagefile) throws IOException {
		Image image=null;
		/*URL imageurl;
		try {
    	    imageurl=new URL(baseurl+imagefile);
		} catch (MalformedURLException e) {
			e.printStackTrace();
			return null;
		}*/
		try {
			image = PnMImageLoader.loadPnM(imagefile);
		} catch (IOException e) { e.printStackTrace(); }
		if (image==null) {
        	System.err.println("Warning: failed to load "+imagefile);
			return null;
		}
		return image;
	}
	static Dimension getImageSize(Image image, ImageObserver o) {
		if (image==null) return new Dimension(1,1);
		while (image.getWidth(o) == -1 || image.getHeight(o) == -1) {}
		return new Dimension(image.getWidth(o), image.getHeight(o));
	}
	static final double PREVIEWSIZE=768;
	static class ImageCanvas extends Canvas implements MouseListener {
		String imagename;
		Image image=null;
		public Dimension imagesize=null;
		public Dimension dispsize=null;
		double xscale,yscale;
		int cx1,cy1,cx2,cy2;
		boolean is_crop;
		public ImageCanvas(boolean is_crop) {
			this.is_crop=is_crop;
			addMouseListener(this);
		}
		void genDispSize() {
			if (imagesize.height > PREVIEWSIZE
			||  imagesize.width  > PREVIEWSIZE ) {
				xscale = imagesize.width / PREVIEWSIZE;
				yscale = imagesize.height / PREVIEWSIZE;
				if (xscale > yscale) {
					yscale=xscale;
				} else {
					xscale=yscale;
				}
				dispsize = new Dimension(
					(int)(imagesize.width/xscale),
					(int)(imagesize.height/yscale) );
			} else {
				dispsize=imagesize;
				xscale=1.0;
				yscale=1.0;
			}
			//System.out.println(dispsize);
		}
		public void setImage(String imagename) {
			this.imagename=imagename;
			reloadImage();
		}
		public void reloadImage() {
			try {
				image=loadImage(imagename);
			} catch (IOException e) {
				e.printStackTrace();
			}
			imagesize = getImageSize(image,this);
			genDispSize();
			setSize(dispsize);
			try {
				((Frame)getParent()).pack();
				repaint();
			} catch (NullPointerException e) {}
			setCrop(0,0,imagesize.width-1,imagesize.height-1);
		}
		public void setCrop(double nx1,double ny1, double nx2,double ny2) {
			cx1=(int)nx1;
			cy1=(int)ny1;
			cx2=(int)nx2;
			cy2=(int)ny2;
			repaint();
		}
		public void paint(Graphics g) {
			if (image!=null) {
				g.drawImage(image,0,0,dispsize.width,dispsize.height,
					Color.black,this);
				if (is_crop) {
					g.setColor(Color.magenta);
					g.setXORMode(Color.white);
					//System.out.println(""+cx1+" "+cy1+" "+(cx2-cx1)+" "+(cy2-cy1));
					g.drawRect((int)(cx1/xscale), (int)(cy1/yscale),
						(int)((cx2-cx1)/xscale), (int)((cy2-cy1)/yscale));
				}
			}
		}
		public Dimension getPreferredSize() {
			return new Dimension(dispsize);
		}
		public void mouseClicked(MouseEvent e) {
			if ((e.getModifiers() & InputEvent.BUTTON1_MASK) != 0) {
				CanvasXYtoCrop(e.getX()*xscale, e.getY()*yscale,
					cx2, cy2);
				//cx1=e.getX();
				//cy1=e.getY();
				//x1.setValue(cx1);
				//y1.setValue(cy1);
			}
			if ((e.getModifiers() & InputEvent.BUTTON3_MASK) != 0) {
				CanvasXYtoCrop(cx1, cy1,
					e.getX()*xscale, e.getY()*yscale);
				//cx2=e.getX();
				//cy2=e.getY();
				//x2.setValue(cx2);
				//y2.setValue(cy2);
			}
		}
		public void mouseEntered(MouseEvent e) {}
		public void mouseExited(MouseEvent e) {}
		public void mousePressed(MouseEvent e) {}
		public void mouseReleased(MouseEvent e) {}
	}
	static String genCropCmd() {
		lcx1=x1.getValue();
		lcy1=y1.getValue();
		lcx2=x2.getValue();
		lcy2=y2.getValue();
		String cmd = "pnmcut";
		cmd += " -left "+ lcx1;
		cmd += " -right "+ lcx2;
		cmd += " -top "+ lcy1;
		cmd += " -bottom "+ lcy2;
		cmd += " <"+source;
		if (rotstate==-1) {
			cmd += " | pnmflip -r270";
		} else if (rotstate == 1) {
			cmd += " | pnmflip -r90";
		}
		if (is_preview) {
			cmd += " | pnmresample  -w 1 -f 0 -b"+
			       " -x "+PREVIEWSIZE+" -y "+PREVIEWSIZE;
		}
		cmd += " >"+cropprv;
		return cmd;
	}
	static void calcAndSetCrop() {
		double xscale=-1,yscale=-1;
		switch (rotstate) {
		case -1 : 
			xscale = croppc.imagesize.height / (double)(lcx2 - lcx1);
			yscale = croppc.imagesize.width / (double)(lcy2 - lcy1);
			croppc.setCrop(
					yscale*(lcy2 - y2.getValue()),
					xscale*(x1.getValue() - lcx1),
					yscale*(lcy2 - y1.getValue()), 
					xscale*(x2.getValue() - lcx1) );
			//croppc.setCrop(
			//		yscale*(y1.getValue()-lcy1),
			//		xscale*(lcx2 - x2.getValue()),
			//		yscale*(y2.getValue()-lcy1), 
			//		xscale*(lcx2 - x1.getValue()) );
		break;
		case 1 : 
			xscale = croppc.imagesize.height / (double)(lcx2 - lcx1);
			yscale = croppc.imagesize.width / (double)(lcy2 - lcy1);
			croppc.setCrop(
					yscale*(y1.getValue() - lcy1),
					xscale*(lcx2 - x2.getValue()),
					yscale*(y2.getValue() - lcy1), 
					xscale*(lcx2 - x1.getValue()) );
			//croppc.setCrop(
			//		yscale*(y2.getValue()-lcy1),
			//		xscale*(x1.getValue()-lcx1),
			//		yscale*(y1.getValue()-lcy1),
			//		xscale*(x2.getValue()-lcx1) );
		break;
		case 0 :
			xscale = croppc.imagesize.width / (double)(lcx2 - lcx1);
			yscale = croppc.imagesize.height / (double)(lcy2 - lcy1);
			croppc.setCrop(
					xscale*(x1.getValue()-lcx1),
					yscale*(y1.getValue()-lcy1),
					xscale*(x2.getValue()-lcx1),
					yscale*(y2.getValue()-lcy1) );
		break;
		}
		//System.out.println(" "+xscale+" "+yscale+" ");
	}
	static void CanvasXYtoCrop(double nx1,double ny1, double nx2,double ny2) {
		double xscale,yscale;
		switch (rotstate) {
		case -1 :
			xscale = croppc.imagesize.height / (double)(lcx2 - lcx1);
			yscale = croppc.imagesize.width / (double)(lcy2 - lcy1);
			x1.setValue((int)(ny1/yscale+lcx1));
			y1.setValue((int)(-nx2/xscale+lcy2));
			x2.setValue((int)(ny2/yscale+lcx1));
			y2.setValue((int)(-nx1/xscale+lcy2));
		break;
		case 1 :
			xscale = croppc.imagesize.height / (double)(lcx2 - lcx1);
			yscale = croppc.imagesize.width / (double)(lcy2 - lcy1);
			x1.setValue((int)(-ny2/yscale+lcx2));
			y1.setValue((int)(nx1/xscale+lcy1));
			x2.setValue((int)(-ny1/yscale+lcx2));
			y2.setValue((int)(nx2/xscale+lcy1));
		break;
		case 0 :
			xscale = croppc.imagesize.width / (double)(lcx2 - lcx1);
			yscale = croppc.imagesize.height / (double)(lcy2 - lcy1);
			x1.setValue((int)(nx1/xscale+lcx1));
			y1.setValue((int)(ny1/yscale+lcy1));
			x2.setValue((int)(nx2/xscale+lcx1));
			y2.setValue((int)(ny2/yscale+lcy1));
		break;
		}
		calcAndSetCrop();
	}
	static class CropListener implements AdjustmentListener {
		public void adjustmentValueChanged(AdjustmentEvent e) {
			calcAndSetCrop();
		}
	}
	static String genHSYCmd() {
		String cmd = "ppmhsy -E";
		cmd += " -s "+ (s.getValue()/100.0);
		cmd += " -y "+ (y.getValue()/100.0);
		cmd += " -Y "+ (Y.getValue()/100.0);
		cmd += " -c "+ (contrast.getValue()/100.0);
		cmd += " -r "+ (1.0+r.getValue()/100.0);
		cmd += " -g "+ (1.0+g.getValue()/100.0);
		cmd += " -b "+ (1.0+b.getValue()/100.0);
		cmd += " -n "+ (n.getValue()/100.0);
		cmd += " -w "+ (whiten.getValue()/100.0);
		for (int i=0; i<6; i++)
			cmd += " -"+i+" "+(hs[i].getValue()/100.0);
		cmd += " <"+cropprv;
		if (sharpen) {
			cmd +=" | pnmconvolve -u -s 12 -b 0.04 -i 2 ";
		}
		cmd +=" >"+hsyprv;
		return cmd;
	}
	static class HSYListener implements AdjustmentListener {
		public void adjustmentValueChanged(AdjustmentEvent e) {
			setHSYLabels();
			String cmd=genHSYCmd();
			exec.execute(cmd,hsypc,1000);
			//System.out.println(""+e.getAdjustable()+e.getValue());
		}
	}
	static String genInitCmd() {
		if (origfile.endsWith(".jpg")
		||  origfile.endsWith(".JPG")
		||  origfile.endsWith(".jpeg")
		||  origfile.endsWith(".JPEG") ) {
			return "jpegtopnm <"+origfile+" >"+source;
		} else {
			return "cp "+origfile+" "+source;
		}
	}

	static class CmdExecuter implements Runnable {
		String cmd=null;
		ImageCanvas observer=null;
			int wait;
		public CmdExecuter () {
			new Thread(this).start();
		}
		public void run() {
			String [] cmdline = {"/bin/sh", "-c", ""};
			ImageCanvas thisobs;
			int thiswait;
			while (true) {
				synchronized (this) {
					/* wait till command arrives */
					while (cmd==null) {
						try { wait(); }
						catch (InterruptedException e) { return; }
					}
				}
				synchronized (this) {
					thiswait=wait;
				}
				/* hang on for awhile, there may be another command */
				try {
					Thread.sleep(wait);
				} catch (InterruptedException e) { return; }
				/* execute latest command */
				synchronized (this) {
					/* execute command */
					cmdline[2] = cmd;
					thisobs = observer;
					cmd=null;
				}
				try {
					System.out.println("exec: "+cmdline[2]);
	   	 	    	Runtime.getRuntime().exec(cmdline).waitFor();
				} catch (Exception e) { e.printStackTrace(); }
				/* reload image */
				thisobs.reloadImage();
			}
		}
		public void execute(String cmd,ImageCanvas observer, int wait) {
			synchronized (this) {
				this.cmd=cmd;
				this.observer=observer;
				this.wait=wait;
				notifyAll();
			}
		}
	}

	static void execSync(String cmd) throws IOException,InterruptedException {
		String [] cmdline = {"/bin/sh", "-c", cmd };
		System.out.println("exec "+cmdline[2]);
		Runtime.getRuntime().exec(cmdline).waitFor();
		//Thread.sleep(500);
	}
	static void resetCrop() {
		x1.setValue(0);
		x2.setValue(imgsize.width);
		y1.setValue(0);
		y2.setValue(imgsize.height);
	}
	static void resetHSY() {
		s.setValue(100);
		y.setValue(100);
		Y.setValue(100);
		contrast.setValue(100);
		r.setValue(0);
		g.setValue(0);
		b.setValue(0);
		n.setValue(0);
		whiten.setValue(100);
		for (int i=0; i<6; i++) hs[i].setValue(100);
		setHSYLabels();
	}
	static void setHSYLabels() {
		slab.setText("saturation "+s.getValue()/100.0+"             ");
		ylab.setText("brightness "+y.getValue()/100.0);
		Ylab.setText("gamma "+Y.getValue()/100.0);
		contrastlab.setText("contrast "+contrast.getValue()/100.0);
		rlab.setText("red.....cyan (R gamma="+(1.0+r.getValue()/100.0)+")");
		glab.setText("green.....purple (G gamma="+(1.0+g.getValue()/100.0)+")");
		blab.setText("blue.....yellow (B gamma="+(1.0+b.getValue()/100.0)+")");
		nlab.setText("noise dampening "+n.getValue()/100.0);
		whitenlab.setText("whitening "+whiten.getValue()/100.0);
		for (int i=0; i<6; i++) hslab[i].setText(hue[i]+" saturation "
			+ hs[i].getValue() / 100.0);
	}
	
	static final int SBSIZE=10;
	static CropListener croplisten = new CropListener();
	static HSYListener hsylisten = new HSYListener();
	public static void main(String [] args)
	throws IOException,InterruptedException {
		if (args.length != 1) {
			System.err.print("\nPlease supply filename of jpg or ppm.\n\n");
			System.exit(0);
		}
		origfile = args[0];
		filenopath = origfile;
		if (origfile.lastIndexOf('/') >= 0) {
			filenopath = 
				origfile.substring(origfile.lastIndexOf('/')+1);
		}
		basename = filenopath;
		if (filenopath.lastIndexOf('.') >= 0) {
			basename = 
				filenopath.substring(0,filenopath.lastIndexOf('.'));
		}
		/* init temp file 0 */
		String cmd = genInitCmd();
		cmd += "; rm "+cropprv+" "+hsyprv;
		execSync(cmd);

		/* set image size and dependent variables */
		imgsize = PnMImageLoader.getPnMSize(source);
		x1 = new Scrollbar(Scrollbar.HORIZONTAL,
			0, SBSIZE, 0, imgsize.width+SBSIZE-1);
		x2 = new Scrollbar(Scrollbar.HORIZONTAL,
			imgsize.width, SBSIZE, 0, imgsize.width+SBSIZE-1);
		y1 = new Scrollbar(Scrollbar.HORIZONTAL,
			0,SBSIZE,0,imgsize.height+SBSIZE-1);
		y2 = new Scrollbar(Scrollbar.HORIZONTAL,
			imgsize.height, SBSIZE, 0, imgsize.height+SBSIZE-1);
		resetCrop();

		/* init temp files 1 and 2 */
		execSync(genCropCmd()); //"cp "+source+" "+cropprv);
		execSync("cp "+cropprv+" "+hsyprv);
		exec=new CmdExecuter();

		cropp.setLayout(new FlowLayout());
		croppc= new ImageCanvas(true);
		croppc.setImage(cropprv);
		cropp.add(croppc);

		hsyp.setLayout(new FlowLayout());
		hsypc= new ImageCanvas(false);
		hsypc.setImage(hsyprv);
		hsyp.add(hsypc);

		crop.add(cropf1);
		crop.add(x1);
		x1.addAdjustmentListener(croplisten);
		crop.add(cropf2);
		crop.add(y1);
		y1.addAdjustmentListener(croplisten);
		crop.add(cropf3);
		crop.add(x2);
		x2.addAdjustmentListener(croplisten);
		crop.add(cropf4);
		crop.add(y2);
		y2.addAdjustmentListener(croplisten);
		crop.add(rotleftbut);
		rotleftbut.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				rotstate=-1;
				cropf1.setText("UP          ^^^^^^^^^^^");
				cropf2.setText("RIGHT       >>>>>>>>>>>");
				cropf3.setText("DOWN        VVVVVVVVVVV");
				cropf4.setText("LEFT        <<<<<<<<<<<");
				calcAndSetCrop();
				String comd = genCropCmd();
				exec.execute(comd,croppc,0);
			}
		} );
		crop.add(rotrightbut);
		rotrightbut.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				rotstate=1;
				cropf1.setText("DOWN        VVVVVVVVVVV");
				cropf2.setText("LEFT        <<<<<<<<<<<");
				cropf3.setText("UP          ^^^^^^^^^^^");
				cropf4.setText("RIGHT       >>>>>>>>>>>");
				calcAndSetCrop();
				String comd = genCropCmd();
				exec.execute(comd,croppc,0);
			}
		} );
		crop.add(norotbut);
		norotbut.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				rotstate=0;
				cropf1.setText("LEFT        <<<<<<<<<<<");
				cropf2.setText("UP          ^^^^^^^^^^^");
				cropf3.setText("RIGHT       >>>>>>>>>>>");
				cropf4.setText("DOWN        VVVVVVVVVVV");
				calcAndSetCrop();
				String comd = genCropCmd();
				exec.execute(comd,croppc,0);
			}
		} );
		crop.add(cropupdbut);
		cropupdbut.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String comd = genCropCmd();
				exec.execute(comd,croppc,0);
			}
		} );
		crop.add(cropresetbut);
		cropresetbut.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				resetCrop();
				String comd = genCropCmd();
				exec.execute(comd,croppc,0);
			}
		} );
		crop.setLayout(new GridLayout(14,1));

		cropp.add(crop);
		cropp.show();
		cropp.pack();
		cropp.pack();

		s = new Scrollbar(Scrollbar.HORIZONTAL, 100, SBSIZE, 0, 500);
		y = new Scrollbar(Scrollbar.HORIZONTAL, 100, SBSIZE, 0, 500);
		Y = new Scrollbar(Scrollbar.HORIZONTAL, 100, SBSIZE, 0, 500);
		contrast = new Scrollbar(Scrollbar.HORIZONTAL, 100, SBSIZE, 0, 500);
		r = new Scrollbar(Scrollbar.HORIZONTAL, 0, SBSIZE, -100, 100+SBSIZE);
		g = new Scrollbar(Scrollbar.HORIZONTAL, 0, SBSIZE, -100, 100+SBSIZE);
		b = new Scrollbar(Scrollbar.HORIZONTAL, 0, SBSIZE, -100, 100+SBSIZE);
		n = new Scrollbar(Scrollbar.HORIZONTAL, 0, SBSIZE, 0, 100+SBSIZE);
		whiten = new Scrollbar(Scrollbar.HORIZONTAL,1000,SBSIZE, 0,100+SBSIZE);
		for (int i=0; i<6; i++) hs[i] = 
			new Scrollbar(Scrollbar.HORIZONTAL, 100, SBSIZE, 0, 500);
		resetHSY();
		setHSYLabels();
		hsy.add(slab);
		hsy.add(s);
		s.addAdjustmentListener(hsylisten);
		hsy.add(ylab);
		hsy.add(y);
		y.addAdjustmentListener(hsylisten);
		hsy.add(Ylab);
		hsy.add(Y);
		Y.addAdjustmentListener(hsylisten);
		hsy.add(contrastlab);
		hsy.add(contrast);
		contrast.addAdjustmentListener(hsylisten);
		hsy.add(rlab);
		hsy.add(r);
		r.addAdjustmentListener(hsylisten);
		hsy.add(glab);
		hsy.add(g);
		g.addAdjustmentListener(hsylisten);
		hsy.add(blab);
		hsy.add(b);
		b.addAdjustmentListener(hsylisten);
		hsy.add(nlab);
		hsy.add(n);
		n.addAdjustmentListener(hsylisten);
		hsy.add(whitenlab);
		hsy.add(whiten);
		whiten.addAdjustmentListener(hsylisten);
		for (int i=0; i<6; i++) {
			hsy.add(hslab[i]);
			hsy.add(hs[i]);
			hs[i].addAdjustmentListener(hsylisten);
		}
		hsy.add(donebut);
		donebut.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					is_preview=false;
					FileWriter fw = new FileWriter(basename+"-fudge.sh");
					fw.write(genInitCmd()+"\n");
					fw.write(genCropCmd()+"\n");
					fw.write(genHSYCmd()+"\n");
					fw.write("cp "+hsyprv+" "+basename+"-fudged.ppm"+"\n");
					fw.close();
				} catch (IOException ex) {
					System.err.println("ERROR: FAILED TO WRITE SCRIPT");
					ex.printStackTrace();
				}
				System.exit(0);
			}
		} );
		hsy.add(sharpbut);
		sharpbut.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				sharpen=true;
				exec.execute(genHSYCmd(),hsypc,0);
				sharpen=false;
			}
		} );
		hsy.add(hsyresetbut);
		hsyresetbut.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				resetHSY();
				exec.execute(genHSYCmd(),hsypc,0);
			}
		} );
		hsy.setLayout(new GridLayout(22+12,1));
		hsyp.add(hsy);
		hsyp.show();
		hsyp.pack();
		hsyp.pack();
	}
}

