/* PnMImageLoader - BufferedImage loader for PnM files.
 *
 * This file is part of PhotoPNMTools,
 * Release $Name:  $,
 * File rev. $Id: PnMImageLoader.java,v 1.3 2003/03/08 17:17:21 schooten Exp $.
 *
 * Copyright (c) 2002, 2003 by B.W. van Schooten. All rights reserved.
 * This software is distributed under the Perl Artistic License.
 * No warranty. See file 'artistic.license' for more details.
 *
 * boris@13thmonkey.org
 * www.13thmonkey.org/~boris/photopnmtools/
 */
import java.awt.*;
import java.awt.image.*;
import java.net.*;
import java.io.*;
public class PnMImageLoader {
	static FileReader in;
	static char ch;
	static boolean comments_allowed;
	public static Dimension getPnMSize(String filename) throws IOException {
		in = new FileReader(filename);
		/* XXX duplicate code */
		comments_allowed=true;
		int type = getPnMMagic();
		getChar();
		int width = getInt();
		int height= getInt();
		in.close();
		return new Dimension(width,height);
	}
	public static Image loadPnM(String filename) throws IOException {
		in = new FileReader(filename);
		/* header */
		comments_allowed=true;
		int type = getPnMMagic();
		getChar();
		int width = getInt();
		int height= getInt();
		int depth = getInt();
		comments_allowed=false;
		BufferedImage i=null;
		while (i==null) {
			try {
				i = new BufferedImage(
					width, height, BufferedImage.TYPE_INT_RGB);
			} catch (RasterFormatException e) {
				e.printStackTrace();
			}
		}
		WritableRaster r = i.getRaster();
		char [] data = new char [width*3];
		int [] pixels = new int [width*3];
		if (type==6) {
			for (int y=0; y<height; y++) {
				int nr = in.read(data,0,data.length);
				if (nr < width*3) {
					System.err.println("WARNING: unfinished line in PnM");
				}
				for (int x=width*3-1; x>0; x--) {
					pixels[x] = data[x];
				}
				r.setPixels(0,y,width,1,pixels);
			}
		} else {
			in.close();
			throw new IOException("Unsupported PnM type "+type);
		}
		in.close();
		return i;
	}
	/** also skip comments if applicable.
	*/
	public static void getChar() throws IOException {
		ch = (char)in.read();
		if (comments_allowed && ch=='#') {
			while (ch!= '\n')
				ch=(char)in.read();
			ch = (char)in.read();
		}
	}
	public static int getPnMMagic() throws IOException {
		getChar();
		if (ch != 'P') throw new IOException ("bad PnM magic bytes");
		getChar();
		if (ch >= '1' && ch <='6')
			return (int) (ch-'0');
		else
			throw new IOException("bad PnM magic bytes");
	}
	public static int getInt() throws IOException {
		String s = "";
		getChar();
		while ( (ch != '\n') && Character.isSpaceChar(ch) ) 
			getChar();
		while ( (ch != '\n') && !Character.isSpaceChar(ch) ) {
			s = s + ch;
			getChar();
		}
		//System.out.println(s);
		return Integer.parseInt(s);
	}
}

