/*  PPMTESTIMAGE - Generate test image for printing
 *
 * This file is part of PhotoPNMTools,
 * Release $Name:  $,
 * File rev. $Id: ppmtestimage.c,v 1.2 2003/02/22 12:21:19 schooten Exp $.
 *
 * Copyright (c) 2002, 2003 by B.W. van Schooten. All rights reserved.
 * This software is distributed under the Perl Artistic License.
 * No warranty. See file 'artistic.license' for more details.
 *
 * boris@13thmonkey.org
 * www.13thmonkey.org/~boris/photopnmtools/
 */

#include <pnm.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#define MAX(a,b) ( (a) > (b) ? (a) : (b) )

#define USAGE \
"ppmtestimage: create test image for printing\n" \
"options: -x xsize\n" \
"         -y ysize\n" \
"\n" \


	int
main(int argc, char **argv) {
	xel **in,**out;

	int xofs,yofs;
	double xval,yval;
	long r,g,b;
	char optch;

	int nr_bands=20;

	pixval maxpix=255;
	int format=20531;
	int xsize=1024,ysize=768;
	int x,y,z;
	pnm_init(&argc,argv);
	while ( (optch = getopt(argc, argv, "x:y:")) != -1) {
		switch (optch) {
		case 'x': xsize=atol(optarg);
		break;
		case 'y': ysize=atol(optarg);
		break;
		}
	}
	argc -= optind;
	argv += optind;
	if (argc != 0) {
		fprintf(stderr,USAGE);
		exit(EXIT_FAILURE);
	}

	//in = pnm_readpnm(stdin,&xsize,&ysize,&maxpix,&format);
	out = pnm_allocarray(xsize,ysize);

	/* colour pattern */
	for (y=0; y<ysize/5; y++) {
		for (x=0; x<xsize/7; x++) {
			xval = ((double)x / (xsize/7));
			yval = ((double)y / (ysize/5));
			out[y][x].r = maxpix*xval;
			out[y][x].g = maxpix*yval;
			out[y][x].b = 0;
		}
	}
	xofs=xsize/6;
	for (y=0; y<ysize/5; y++) {
		for (x=0; x<xsize/7; x++) {
			xval = ((double)x / (xsize/7));
			yval = ((double)y / (ysize/5));
			out[y][xofs+x].r = maxpix*xval;
			out[y][xofs+x].g = maxpix*yval;
			out[y][xofs+x].b = maxpix;
		}
	}
	xofs=2*xsize/6;
	for (y=0; y<ysize/5; y++) {
		for (x=0; x<xsize/7; x++) {
			xval = ((double)x / (xsize/7));
			yval = ((double)y / (ysize/5));
			out[y][xofs+x].r = 0;
			out[y][xofs+x].g = maxpix*xval;
			out[y][xofs+x].b = maxpix*yval;
		}
	}
	xofs=3*xsize/6;
	for (y=0; y<ysize/5; y++) {
		for (x=0; x<xsize/7; x++) {
			xval = ((double)x / (xsize/7));
			yval = ((double)y / (ysize/5));
			out[y][xofs+x].r = maxpix;
			out[y][xofs+x].g = maxpix*xval;
			out[y][xofs+x].b = maxpix*yval;
		}
	}
	xofs=4*xsize/6;
	for (y=0; y<ysize/5; y++) {
		for (x=0; x<xsize/7; x++) {
			xval = ((double)x / (xsize/7));
			yval = ((double)y / (ysize/5));
			out[y][xofs+x].r = maxpix*xval;
			out[y][xofs+x].g = 0;
			out[y][xofs+x].b = maxpix*yval;
		}
	}
	xofs=5*xsize/6;
	for (y=0; y<ysize/5; y++) {
		for (x=0; x<xsize/7; x++) {
			xval = ((double)x / (xsize/7));
			yval = ((double)y / (ysize/5));
			out[y][xofs+x].r = maxpix*xval;
			out[y][xofs+x].g = maxpix;
			out[y][xofs+x].b = maxpix*yval;
		}
	}


	yofs = ysize/4;

	xofs=0*xsize/6;
	for (y=0; y<ysize/5; y++) {
		for (x=0; x<xsize/7; x++) {
			xval = ((double)x / (xsize/7));
			yval = ((double)y / (ysize/5));
			out[yofs+y][xofs+x].r = (maxpix-50)*xval + random()%50;
			out[yofs+y][xofs+x].g = (maxpix-50)*yval + random()%50;
			out[yofs+y][xofs+x].b =                    random()%50;
		}
	}
	xofs=1*xsize/6;
	for (y=0; y<ysize/5; y++) {
		for (x=0; x<xsize/7; x++) {
			xval = ((double)x / (xsize/7));
			yval = ((double)y / (ysize/5));
			out[yofs+y][xofs+x].r = (maxpix-50)*xval + random()%50;
			out[yofs+y][xofs+x].g = (maxpix-50)*yval + random()%50;
			out[yofs+y][xofs+x].b = (maxpix-50)      + random()%50;
		}
	}
	xofs=2*xsize/6;
	for (y=0; y<ysize/5; y++) {
		for (x=0; x<xsize/7; x++) {
			xval = ((double)x / (xsize/7));
			yval = ((double)y / (ysize/5));
			out[yofs+y][xofs+x].r =                    random()%50;
			out[yofs+y][xofs+x].g = (maxpix-50)*xval + random()%50;
			out[yofs+y][xofs+x].b = (maxpix-50)*yval + random()%50;
		}
	}
	xofs=3*xsize/6;
	for (y=0; y<ysize/5; y++) {
		for (x=0; x<xsize/7; x++) {
			xval = ((double)x / (xsize/7));
			yval = ((double)y / (ysize/5));
			out[yofs+y][xofs+x].r = (maxpix-50)      + random()%50;
			out[yofs+y][xofs+x].g = (maxpix-50)*xval + random()%50;
			out[yofs+y][xofs+x].b = (maxpix-50)*yval + random()%50;
		}
	}
	xofs=4*xsize/6;
	for (y=0; y<ysize/5; y++) {
		for (x=0; x<xsize/7; x++) {
			xval = ((double)x / (xsize/7));
			yval = ((double)y / (ysize/5));
			out[yofs+y][xofs+x].r = (maxpix-50)*xval + random()%50;
			out[yofs+y][xofs+x].g =                    random()%50;
			out[yofs+y][xofs+x].b = (maxpix-50)*yval + random()%50;
		}
	}
	xofs=5*xsize/6;
	for (y=0; y<ysize/5; y++) {
		for (x=0; x<xsize/7; x++) {
			xval = ((double)x / (xsize/7));
			yval = ((double)y / (ysize/5));
			out[yofs+y][xofs+x].r = (maxpix-50)*xval + random()%50;
			out[yofs+y][xofs+x].g = (maxpix-50)      + random()%50;
			out[yofs+y][xofs+x].b = (maxpix-50)*yval + random()%50;
		}
	}
	xofs=4*xsize/6;
	for (y=0; y<ysize/5; y++) {
		for (x=0; x<xsize/7; x++) {
			xval = ((double)x / (xsize/7));
			yval = ((double)y / (ysize/5));
			out[yofs+y][xofs+x].r = (maxpix-50)*xval + random()%50;
			out[yofs+y][xofs+x].g =                    random()%50;
			out[yofs+y][xofs+x].b = (maxpix-50)*yval + random()%50;
		}
	}
	xofs=5*xsize/6;
	for (y=0; y<ysize/5; y++) {
		for (x=0; x<xsize/7; x++) {
			xval = ((double)x / (xsize/7));
			yval = ((double)y / (ysize/5));
			out[yofs+y][xofs+x].r = (maxpix-50)*xval + random()%50;
			out[yofs+y][xofs+x].g = (maxpix-50)      + random()%50;
			out[yofs+y][xofs+x].b = (maxpix-50)*yval + random()%50;
		}
	}
	/* line patterns */
	for (z=1; z<nr_bands; z++) {
		xofs=z*xsize/nr_bands;
		yofs=ysize/2;
		for (y=0; y<ysize/8; y++) {
			for (x=0; x<xsize/nr_bands; x++) {
				out[yofs+y][xofs+x].r = (maxpix/2) +
					(maxpix/2)*cos((1.0/z)*((float)x*3.1415265));
				out[yofs+y][xofs+x].g = (maxpix/2) +
					(maxpix/2)*cos((1.0/z)*((float)x*3.1415265));
				out[yofs+y][xofs+x].b = (maxpix/2) +
					(maxpix/2)*cos((1.0/z)*((float)x*3.1415265));
			}
		}
	}
	for (z=1; z<nr_bands; z++) {
		xofs=z*xsize/nr_bands;
		yofs=ysize/2 + ysize/8;
		for (y=0; y<ysize/8; y++) {
			for (x=0; x<xsize/nr_bands; x++) {
				out[yofs+y][xofs+x].r = (maxpix/2) +
					(maxpix/2)*cos((1.0/z)*((float)(x+y)*3.1415265));
				out[yofs+y][xofs+x].g = (maxpix/2) +
					(maxpix/2)*cos((1.0/z)*((float)(x+y)*3.1415265));
				out[yofs+y][xofs+x].b = (maxpix/2) +
					(maxpix/2)*cos((1.0/z)*((float)(x+y)*3.1415265));
			}
		}
	}
	for (z=1; z<nr_bands; z++) {
		xofs=z*xsize/nr_bands;
		yofs=ysize/2 + 2*ysize/8;
		for (y=0; y<ysize/8; y++) {
			for (x=0; x<xsize/nr_bands; x++) {
				out[yofs+y][xofs+x].r = (maxpix/2) +
					(maxpix/2)*cos((1.0/z)*((float)(y)*3.1415265));
				out[yofs+y][xofs+x].g = (maxpix/2) +
					(maxpix/2)*cos((1.0/z)*((float)(y)*3.1415265));
				out[yofs+y][xofs+x].b = (maxpix/2) +
					(maxpix/2)*cos((1.0/z)*((float)(y)*3.1415265));
			}
		}
	}
	for (z=1; z<nr_bands; z++) {
		xofs=z*xsize/nr_bands;
		yofs=ysize/2 + 3*ysize/8;
		for (y=0; y<ysize/8; y++) {
			for (x=0; x<xsize/nr_bands; x++) {
				out[yofs+y][xofs+x].r = (maxpix/2) +
					(maxpix/2)*cos((1.0/z)*((float)(x-y)*3.1415265));
				out[yofs+y][xofs+x].g = (maxpix/2) +
					(maxpix/2)*cos((1.0/z)*((float)(x-y)*3.1415265));
				out[yofs+y][xofs+x].b = (maxpix/2) +
					(maxpix/2)*cos((1.0/z)*((float)(x-y)*3.1415265));
			}
		}
	}

	pnm_writepnm(stdout,out,xsize,ysize,maxpix,format,0/*forceplain*/);
	//pnm_freearray(in,ysize);
	pnm_freearray(out,ysize);
	exit(EXIT_SUCCESS);
	return 0;
}

